﻿Imports System.Activities
Imports OfficeInterfaces

''' <summary>
''' A Workflow Activity to fill in named ranges in an Excel Worksheet from input parameters
''' </summary>
''' <remarks>
''' Matches values in named fields to named ranges
''' </remarks>
Public NotInheritable Class ExcelFormActivity
    Inherits CodeActivity
    'Define Arguments
    Property Fields As InArgument(Of Dictionary(Of String, String))
    Property InputDocument As InArgument(Of String)
    Property OutputDocument As InArgument(Of String)
    Private InputFields As Dictionary(Of String, String)
    Private InputDocumentName As String
    Private OutputDocumentName As String
    Private TargetActivityName As String
    ''' <summary>
    ''' Fill in fields in the input document, save to the output document
    ''' if the input Fields contains a document of type xlsx or this activity is named as the TargetActivity
    '''--------------------------------------------------------------------------------------------
    '''
    ''' Input Document path comes from (in order of preference):
    '''   1. The "InputDocument" workflow property
    '''   2. A Field named InputDocument
    '''--------------------------------------------------------------------------------------------
    ''' </summary>
    ''' <param name="context"></param>
    ''' <remarks></remarks>
    Protected Overrides Sub Execute(ByVal context As CodeActivityContext)
        InputFields = Fields.Get(Of Dictionary(Of String, String))(context)
        If InputFields Is Nothing Then
            MessageBox.Show("No Input Data - exiting", Me.DisplayName)
            Exit Sub
        End If
        InputDocumentName = InputDocument.Get(Of String)(context)
        If String.IsNullOrEmpty(InputDocumentName) Then InputDocumentName = InputFields("InputDocument")

        OutputDocumentName = OutputDocument.Get(Of String)(context)
        If String.IsNullOrEmpty(OutputDocumentName) Then OutputDocumentName = InputFields("OutputDocument")

        ' Test to see if this is the right activity to process the input document
        InputFields.TryGetValue(("TargetActivity"), TargetActivityName)
        ' If there is no explicit target, see if the document is the right type
        If String.IsNullOrEmpty(TargetActivityName) Then
            If Not (InputDocumentName.EndsWith(".xlsx") Or InputDocumentName.EndsWith(".xlsm")) Then Exit Sub
            'If this is the Target Activity, fix the document name as needed  
        ElseIf TargetActivityName = Me.DisplayName Then
            If Not (InputDocumentName.EndsWith(".xlsx") Or InputDocumentName.EndsWith(".xlsm")) Then InputDocumentName &= ".xlsx"
            If Not System.IO.File.Exists(InputDocumentName) Then
                MessageBox.Show(String.Format("Input Document {0} not found"), InputDocumentName)
                Exit Sub
            End If
        Else
            Exit Sub
        End If

        ' This is the target activity, or there is no explicit target and the input document is an Excel document
        Dim InputExcelInterface = New ExcelInterface(InputDocumentName, InputFields)
        'If the input document has macros, you must name the output document ending with ".xlsm" in order to open it.
        If Not (OutputDocumentName.EndsWith(".xlsx") Or OutputDocumentName.EndsWith(".xlsm")) Then OutputDocumentName &= ".xlsx"
        InputExcelInterface.SaveAs = OutputDocumentName
        Dim Result As Boolean = InputExcelInterface.FillInDocument()

        ' Display the resulting document
        System.Diagnostics.Process.Start(OutputDocumentName)
    End Sub

End Class
