﻿Imports System.Activities
Imports System.ComponentModel
Imports OfficeInterfaces

<Designer(GetType(WordFormActivityDesigner))> _
Public NotInheritable Class WordFormActivity
    Inherits CodeActivity
    'Define Arguments
    Public Property Fields As InArgument(Of Dictionary(Of String, String))
    Public Property InputDocument As InArgument(Of String)
    Public Property OutputDocument As InArgument(Of String)

    Private InputFields As Dictionary(Of String, String)
    Private InputDocumentName As String
    Private OutputDocumentName As String
    Private TargetActivityName As String

    ''' <summary>
    ''' Fill in fields in the input document, save to the output document
    ''' if the input Fields contains a document of type docx or docm or this activity is named as the TargetActivity
    '''--------------------------------------------------------------------------------------------
    ''' Input Document path comes from (in order of preference):
    '''   1. The "InputDocument" workflow property
    '''   2. A Field named InputDocument
    '''--------------------------------------------------------------------------------------------
    ''' </summary>
    ''' <param name="context"></param>
    ''' <remarks></remarks>
    Protected Overrides Sub Execute(ByVal context As CodeActivityContext)
        InputFields = Fields.Get(Of Dictionary(Of String, String))(context)
        If InputFields Is Nothing Then
            MessageBox.Show("No Input Data - exiting", Me.DisplayName)
            Exit Sub
        End If
        InputDocumentName = InputDocument.Get(Of String)(context)
        If String.IsNullOrEmpty(InputDocumentName) Then InputDocumentName = InputFields("InputDocument")

        OutputDocumentName = OutputDocument.Get(Of String)(context)
        If String.IsNullOrEmpty(OutputDocumentName) Then OutputDocumentName = InputFields("OutputDocument")

        ' Test to see if this is the right activity to process the input document
        InputFields.TryGetValue(("TargetActivity"), TargetActivityName)
        ' If there is no explicit target, see if the document is the right type
        If String.IsNullOrEmpty(TargetActivityName) Then
            If Not (InputDocumentName.EndsWith(".docx") Or InputDocumentName.EndsWith(".docm")) Then Exit Sub
            'If this is the Target Activity, fix the document name as needed
        ElseIf TargetActivityName = Me.DisplayName Then
            If Not (InputDocumentName.EndsWith(".docx") Or InputDocumentName.EndsWith(".docm")) Then InputDocumentName &= ".docx"
            If Not System.IO.File.Exists(InputDocumentName) Then
                MessageBox.Show(String.Format("Input Document {0} not found"), InputDocumentName)
                Exit Sub
            End If
        Else
            Exit Sub
        End If

        ' This is the target activity, or there is no explicit target and the input document is a Word document
        Dim InputWordInterface = New WordInterface(InputDocumentName, InputFields)
        If Not (OutputDocumentName.EndsWith(".docx") Or OutputDocumentName.EndsWith(".docm")) Then OutputDocumentName &= ".docx"
        InputWordInterface.SaveAs = OutputDocumentName
        Dim Result As Boolean = InputWordInterface.FillInDocument()

        ' Display the resulting document
        System.Diagnostics.Process.Start(OutputDocumentName)
        'Dim StartInfo As New System.Diagnostics.ProcessStartInfo(OutputDocumentName)
        'StartInfo.Verb = "print"
        'System.Diagnostics.Process.Start(StartInfo)
    End Sub
End Class
