﻿Imports System.Collections.Generic
Imports System.Globalization
Imports Microsoft.VisualStudio.TestTools.UnitTesting
Imports OfficeInterfaces
Imports System.Configuration.ConfigurationManager
'''<summary>
'''This is a test class for ExcelInterfaceTest and is intended
'''to contain all ExcelInterfaceTest Unit Tests
'''</summary>
<TestClass()> _
Public Class ExcelInterfaceTests

    Private TemplatePath As String
    Private ExcelDocumentName As String
    Private testContextInstance As TestContext
    Private TestExcelInterface As ExcelInterface
    Private Fields As New Dictionary(Of String, String)
    Private OutputPath As String

    <TestInitialize()>
    Public Sub TestInitialize()
        Fields.Clear()
        TestExcelInterface = New ExcelInterface
        For Each Key As String In AppSettings.Keys
            Select Case Key
                Case "ExcelDocument"
                    ExcelDocumentName = AppSettings(Key)
                Case "TemplatePath"
                    TemplatePath = AppSettings(Key)
                Case "OutputPath"
                    OutputPath = AppSettings(Key)
                Case Else
                    Fields.Add(Key, AppSettings(Key))
            End Select
        Next Key
    End Sub
    <TestCleanup()>
    Public Sub TestCleanup()
        TestExcelInterface.Dispose()
        TestExcelInterface = Nothing
    End Sub
    '''<summary>
    '''Gets or sets the test context which provides
    '''information about and functionality for the current test run.
    '''</summary>
    Public Property TestContext() As TestContext
        Get
            Return testContextInstance
        End Get
        Set(ByVal value As TestContext)
            testContextInstance = Value
        End Set
    End Property
    '''<summary>
    '''A test for FillInDocument
    '''</summary>
    <TestMethod()> _
    Public Sub FillInDocumentTestExcel()
        Using TestExcelInterface As New ExcelInterface(TemplatePath & ExcelDocumentName, Fields)
            Dim Title As String = "Output"
            If Fields.ContainsKey("LastName") AndAlso Not String.IsNullOrEmpty(Fields("LastName")) Then Title = Fields("LastName")
            TestExcelInterface.SpreadSheetDocument.PackageProperties.Title = Title
            Dim OutputFile As String = OutputPath & TestExcelInterface.SpreadSheetDocument.PackageProperties.Title & ".xlsx"
            TestExcelInterface.SaveAs = OutputFile
            Dim Result As Boolean = TestExcelInterface.FillInDocument()
            Assert.IsTrue(Result)
            Using CheckExcelInterface As New ExcelInterface(OutputFile, Fields)
                Dim RangeNames As Dictionary(Of String, String) = CheckExcelInterface.GetFieldNames()
                For Each RangeName As KeyValuePair(Of String, String) In RangeNames
                    If Fields.ContainsKey(RangeName.Key) Then
                        Assert.AreEqual(RangeNames(RangeName.Key), Fields(RangeName.Key))
                    End If
                Next
            End Using
        End Using
    End Sub
    '''<summary>
    '''A test for GetFieldNames
    '''</summary>
    <TestMethod()> _
    Public Sub GetFieldNamesTestExcel()
        Using TestExcelInterface As New ExcelInterface(TemplatePath & ExcelDocumentName)
            Dim RangeNames As Dictionary(Of String, String) = TestExcelInterface.GetFieldNames(TemplatePath & ExcelDocumentName)
            Assert.IsNotNull(RangeNames)
            Assert.IsTrue(RangeNames.Count > 0)
        End Using
    End Sub
    <TestMethod()>
    Public Sub WriteExcelDocument()
        Using TestExcelInterface As New ExcelInterface(TemplatePath & ExcelDocumentName, Fields)
            Dim Result As Boolean = TestExcelInterface.FillInDocument()
            Assert.IsNotNull(Result)
            Dim Title As String = "Output"
            If Fields.ContainsKey("LastName") AndAlso Not String.IsNullOrEmpty(Fields("LastName")) Then Title = Fields("LastName")
            TestExcelInterface.SpreadSheetDocument.PackageProperties.Title = Title
            TestExcelInterface.SaveDocument(OutputPath & TestExcelInterface.SpreadSheetDocument.PackageProperties.Title & ".xlsx")
        End Using
    End Sub
    <TestMethod()>
    Public Sub ExcelDateConversions()
        Using TestExcelInterface As New ExcelInterface(TemplatePath & ExcelDocumentName, Fields)
           Dim ExcelDate As String = TestExcelInterface.ConvertDateTimeToExcelDate(DateTime.Parse(Fields("Date"), CultureInfo.CurrentCulture))
            Dim ReturnedDate As DateTime = TestExcelInterface.ConvertExcelDateToDateTime(ExcelDate)
            Assert.AreEqual(DateTime.Parse(Fields("Date")), ReturnedDate)
        End Using
    End Sub
    <TestMethod()>
    Public Sub GetRangeLocation()
        Using TestExcelInterface As New ExcelInterface(TemplatePath & ExcelDocumentName)
            Dim RangeName As String = String.Empty
            If Fields.ContainsKey("Name") AndAlso Not String.IsNullOrEmpty(Fields("LastName")) Then RangeName = "Name"
            Dim RangeLocation As String = TestExcelInterface.GetRangeLocation(RangeName)
            Assert.IsNotNull(RangeLocation)
        End Using
    End Sub
End Class
