﻿Imports OfficeInterfaces
Imports System.Configuration.ConfigurationManager
<TestClass()>
Public Class WordInterfaceTests
    Private TemplatePath As String
    Private WordDocumentName As String
    Private WordTableDocumentName As String
    Private TestWordInterface As WordInterface
    Private Fields As New Dictionary(Of String, String)
    Private OutputPath As String
#Region "Additional test attributes"
    '
    'You can use the following additional attributes as you write your tests:
    '
    'Use ClassInitialize to run code before running the first test in the class
    '<ClassInitialize()>  _
    'Public Shared Sub MyClassInitialize(ByVal testContext As TestContext)
    'End Sub
    '
    'Use ClassCleanup to run code after all tests in a class have run
    '<ClassCleanup()>  _
    'Public Shared Sub MyClassCleanup()
    'End Sub
    '
    'Use TestInitialize to run code before running each test
    '<TestInitialize()>  _
    'Public Sub MyTestInitialize()
    'End Sub
    '
    'Use TestCleanup to run code after each test has run
    '<TestCleanup()>  _
    'Public Sub MyTestCleanup()
    'End Sub
    '
#End Region
    <TestInitialize()>
    Public Sub TestInitialize()
        Fields.Clear()
        'TestWordInterface = New WordInterface
        For Each Key As String In AppSettings.Keys
            Select Case Key
                Case "WordDocument"
                    WordDocumentName = AppSettings(Key)
                Case "TemplatePath"
                    TemplatePath = AppSettings(Key)
                Case "OutputPath"
                    OutputPath = AppSettings(Key)
                Case Else
                    Fields.Add(Key, AppSettings(Key))
            End Select
        Next Key
    End Sub
    <TestCleanup()>
    Public Sub TestCleanup()
        TestWordInterface.Close()
        TestWordInterface.Dispose()
        TestWordInterface = Nothing
    End Sub
    <TestMethod()>
    Public Sub GetBookmarkNamesFromWordMLDocument()
        Debug.WriteLine("Start Test GetBookmarkNamesFromWordMLDocument")
        TestWordInterface = New WordInterface
        Dim BookMarkNames As Dictionary(Of String, String) = TestWordInterface.GetFieldNames(TemplatePath & WordDocumentName)
        TestWordInterface.Close()
        Debug.WriteLine("Bookmark Name, Bookmark Value")
        For Each BookMarkName As KeyValuePair(Of String, String) In BookMarkNames
            If BookMarkName.Key.StartsWith("_") Then Continue For
            Debug.WriteLine("{0}, {1}", BookMarkName.Key, BookMarkName.Value)
        Next
        Assert.IsNotNull(BookMarkNames)
        Assert.IsTrue(BookMarkNames.Count > 0)
        Debug.WriteLine("End Test GetBookmarkNamesFromWordMLDocument")
    End Sub
    <TestMethod()>
    Public Sub FillInWordDocument()
        Debug.WriteLine("Start Test FillInWordDocument")
        TestWordInterface = New WordInterface(TemplatePath & WordDocumentName, Fields)
        FillInWord()
        Debug.WriteLine("End Test FillInWordDocument")
    End Sub

    Private Sub FillInWord()
        Dim Title As String = "Output"
        If Fields.ContainsKey("LastName") AndAlso Not String.IsNullOrEmpty(Fields("LastName")) Then Title = Fields("LastName")
        TestWordInterface.WordDocument.PackageProperties.Title = Title
        TestWordInterface.SaveAs = Me.OutputPath & Title & ".docx"
        Dim Result As Boolean = TestWordInterface.FillInDocument()
        Assert.IsTrue(Result)
        Dim WordDocument = TestWordInterface.WordDocument
        Dim BookMarkNames As Dictionary(Of String, String) = TestWordInterface.GetFieldNames()
        ' Display the resulting document
        System.Diagnostics.Process.Start(Me.OutputPath & Title & ".docx")
        TestWordInterface.Close()
        Assert.IsNotNull(BookMarkNames)
        Assert.IsTrue(BookMarkNames.Count > 0)
        For Each Field As KeyValuePair(Of String, String) In Fields
            If BookMarkNames.Keys.Contains(Field.Key) Then Assert.AreEqual(Fields(Field.Key), BookMarkNames(Field.Key))
        Next
    End Sub
    <TestMethod()>
    Public Sub GetBuiltInDocumentPropertiesWord()
        Debug.WriteLine("Start Test GetBuiltInDocumentPropertiesWord")
        TestWordInterface = New WordInterface(TemplatePath & WordDocumentName, Fields)
        TestWordInterface.GetBuiltInDocumentProperties()
        Assert.IsNotNull(TestWordInterface.BuiltInDocumentProperties)
        Debug.WriteLine("Property Name, Property Value")
        For Each DocumentProperty As KeyValuePair(Of String, String) In TestWordInterface.BuiltInDocumentProperties
            Debug.WriteLine("{0}, {1}", DocumentProperty.Key, DocumentProperty.Value)
        Next
        TestWordInterface.Close()
        Debug.WriteLine("End Test GetBuiltInDocumentPropertiesWord")
    End Sub
    <TestMethod()>
    Public Sub WriteWordDocument()
        TestWordInterface = New WordInterface(TemplatePath & WordDocumentName, Fields)
        Dim Result As Boolean = TestWordInterface.FillInDocument()
        Assert.IsTrue(Result)
        TestWordInterface.SaveDocument(OutputPath & TestWordInterface.WordDocument.PackageProperties.Title & ".docx")
    End Sub
End Class
